@@ -1,15 +1,12 @@
 
 set.seed(290875)
-gctorture(on = FALSE)
-library(party)
-if (!require(mvtnorm))
+library("party")
+if (!require("mvtnorm"))
     stop("cannot load package mvtnorm")
 
-gctorture(on = GCtorture)
 
 ### get rid of the NAMESPACE
-nsparty <- attach(NULL, name="ns-party")
-.Internal(lib.fixup(asNamespace("party"), nsparty))
+attach(asNamespace("party"))
 
 ### 
 ###
